@@ -25,11 +25,43 @@ from cubicweb.dataimport import SQLGenObjectStore
 from cubes.brainomics.importers.helpers import (get_image_info, import_genes,
                                                 import_chromosomes, import_snps)
 
+#http://www.housemd-guide.com/characters/houserules.php
+DR_HOUSE_QUOTES = (u"We treat it. If she[he] gets better we know that we're right.",
+                   u"Patients always want proof, we're not making cars here, we don't give guarantees.",
+                   u"Pretty much all the drugs I prescribe are addictive and dangerous.",
+                   u"Patients sometimes get better. You have no idea why, but unless you give a reason they won't pay you. Anybody notice if there's a full moon? ... let's rule out the lunar god and go from there.",
+                   u"Occam's Razor.  The simplest explanation is almost always somebody screwed up.",
+                   u"I take risks, sometimes patients die. But not taking risks causes more patients to die, so I guess my biggest problem is I've been cursed with the ability to do the math.",
+                   u"Idiopathic, from the Latin meaning we're idiots cause we can't figure out what's causing it.",
+                   u"If he gets better, I'm right, if he dies, you're right.",
+                   u"The eyes can mislead, the smile can lie, but the shoes always tell the truth.",
+                   u"Hang up a shingle and condemn the narrowness and greed of Western medicine, you'd make a damn fine living.",
+                   u"It is the nature of medicine that you are going to screw up.",
+                   u"Right and wrong do exist. Just because you don't know what the right answer is — maybe there's even no way you could know what the right answer is — doesn't make your answer right or even okay. It's much simpler than that. It's just plain wrong.",
+                   u"You know what's worse than useless? Useless and oblivious.",
+                   u"It is in the nature of medicine that you are gonna screw up. You are gonna kill someone. If you can't handle that reality, pick another profession. Or finish medical school and teach.",
+                   u"If it works, we're right. If he dies, it was something else.",
+                   u"If her DNA was off by one percentage point she'd be a dolphin.",)
+
+DR_HOUSE_QUOTES_CONCLUSIONS = (u"Never met a diagnostic study I couldn't refute.",
+                               u"Never trust doctors.",
+                               u"That's a catchy diagnosis, you could dance to that.",
+                               u"Tragedies happen.",
+                               u"Weird works for me.",
+                               u"In case I'm wrong. It has happened.",
+                               u"It does tell us something. Though I have no idea what.",
+                               u"I solved the case, my work is done.",
+                               u"Tests take time. Treatment's quicker.",
+                               u"Welcome to the end of the thought process.",
+                               u"Sometimes we can't see why normal isn't normal.",
+                               u"You want to make things right? Too bad. Nothing's ever right.",
+                               u"New is good. Because old ended in death.")
+
 
 ###############################################################################
 ### DATA GENERATION CONFIGURATION #############################################
 ###############################################################################
-NB_SUBJECTS = 200
+NB_SUBJECTS = 1000
 NB_SUBJECTS_GROUPS = 5
 AGE_DATETIMES = (datetime.strptime('1/1/1930 1:30 PM', '%m/%d/%Y %I:%M %p'),
                  datetime.strptime('1/1/1990 4:50 AM', '%m/%d/%Y %I:%M %p'))
@@ -59,25 +91,29 @@ def random_date(_type=None, start=None, end=None):
     random_second = randrange(int_delta)
     return (start + timedelta(seconds=random_second))
 
-def import_questionnaire(store, subject_eid, center_eid, study_eid,
-                         questionnaire_eid, questions, label):
+def create_protocol(store, study, identifier):
+    """ Create a protocol """
     date = random_date()
-    protocol = store.create_entity('Protocol', identifier=u'Demo questionnaire protocol',
-                                   start_datetime=date)
-    store.relate(protocol.eid, 'related_study', study_eid, subtype='Protocol')
-    assessment = store.create_entity('Assessment', identifier=u'%s_%s' % (label, subject_eid),
-                                     datetime=date)
-    store.relate(assessment.eid, 'protocols', protocol.eid, subtype='Assessment')
-    store.relate(assessment.eid, 'related_study', study_eid, subtype='Assessment')
+    protocol_eid = store.create_entity('Protocol', identifier=identifier,
+                                       start_datetime=date).eid
+    store.relate(protocol_eid, 'related_study', study, subjtype='Protocol')
+    return protocol_eid
+
+def import_questionnaire(store, subject_eid, center_eid, study_eid,
+                         questionnaire_eid, questions, label, protocol_eid):
+    date = random_date()
+    assessment = store.create_entity('Assessment', datetime=date)
+    store.relate(assessment.eid, 'protocols', protocol_eid, subjtype='Assessment')
+    store.relate(assessment.eid, 'related_study', study_eid, subjtype='Assessment')
     store.relate(center_eid, 'holds', assessment.eid)
     store.relate(subject_eid, 'concerned_by', assessment.eid)
     user_ident = nr.randint(10)
     user_ident = u'parent' if user_ident>8 else u'subject'
-    q_qrun = store.create_entity('QuestionnaireRun', identifier=u'%s_%s' % (label, subject_eid),
+    q_qrun = store.create_entity('QuestionnaireRun',
                                    user_ident=user_ident, datetime=date,
                                    iteration=1, completed=True, valid=True,
                                    instance_of=questionnaire_eid)
-    store.relate(q_qrun.eid, 'related_study', study_eid)
+    store.relate(q_qrun.eid, 'related_study', study_eid, subjtype='QuestionnaireRun')
     store.relate(q_qrun.eid, 'concerns', subject_eid, subjtype='QuestionnaireRun')
     store.relate(assessment.eid, 'generates', q_qrun.eid, subjtype='Assessment')
     for name, (question, _min, _max) in questions.iteritems():
@@ -85,22 +121,19 @@ def import_questionnaire(store, subject_eid, center_eid, study_eid,
                                           question=question, datetime=date,
                                           questionnaire_run=q_qrun.eid)
 
-def import_genomic(store, subject_eid, center_eid, study_eid, platform=None, type='SNP'):
+def import_genomic(store, subject_eid, center_eid, study_eid, platform, type, protocol_eid):
     date = random_date()
-    protocol = store.create_entity('Protocol', identifier=u'Demo genomics protocol',
-                                   start_datetime=date)
-    store.relate(protocol.eid, 'related_study', study_eid, subtype='Protocol')
-    assessment = store.create_entity('Assessment', identifier=u'genomics_%s' % subject_eid,
-                                     datetime=date)
-    store.relate(assessment.eid, 'protocols', protocol.eid, subtype='Assessment')
+    assessment = store.create_entity('Assessment', datetime=date)
+    store.relate(assessment.eid, 'protocols', protocol_eid, subjtype='Assessment')
     store.relate(assessment.eid, 'related_study', study_eid, subjtype='Assessment')
     store.relate(center_eid, 'holds', assessment.eid)
     store.relate(subject_eid, 'concerned_by', assessment.eid)
-    measure = store.create_entity('GenomicMeasure', identifier=u'genomic_measure_%s' % subject_eid,
-                                  type=type,
+    measure = store.create_entity('GenomicMeasure', type=type,
                                   platform=platform if platform else None,
-                                  filepath=u'%s/genomics.tar.gz' % subject_eid,
                                   completed=True, valid=True, related_study=study_eid)
+    _file = store.create_entity('ExternalFile', name='genomic_%s' % subject_eid,
+                                absolute_path=False, filepath=u'%s/genomics.tar.gz' % subject_eid)
+    store.relate(measure.eid, 'results_files', _file.eid)
     store.relate(measure.eid, 'concerns', subject_eid, subjtype='GenomicMeasure')
     store.relate(assessment.eid, 'generates', measure.eid, subjtype='Assessment')
     return measure.eid
@@ -120,9 +153,10 @@ def create_genomic_region(store):
         store.relate(entity.eid, 'genes', gene[0], subjtype='GenomicRegion')
     return entity.eid
 
-def import_cgh_results(store, subject_eid, center_eid, study_eid, platform):
+def import_cgh_results(store, subject_eid, center_eid, study_eid, platform, protocol_eid):
     """ Add CGH results """
-    measure = import_genomic(store, subject_eid, center_eid, study_eid, platform, u'CGH')
+    measure = import_genomic(store, subject_eid, center_eid, study_eid,
+                             platform, u'CGH', protocol_eid)
     for region in range(nr.randint(100)):
         region = create_genomic_region(store)
         store.create_entity('CghResult', cgh_ratio=nr.random(), log2_ratio=nr.random(),
@@ -130,9 +164,10 @@ def import_cgh_results(store, subject_eid, center_eid, study_eid, platform):
                             numprobes=nr.randint(100),
                             related_measure=measure, genomic_region=region)
 
-def import_sequencing_results(store, subject_eid, center_eid, study_eid, platform):
+def import_sequencing_results(store, subject_eid, center_eid, study_eid, platform, protocol_eid):
     """ Add CGH results """
-    measure = import_genomic(store, subject_eid, center_eid, study_eid, platform, u'Sequencing')
+    measure = import_genomic(store, subject_eid, center_eid, study_eid,
+                             platform, u'Sequencing', protocol_eid)
     for mutation in range(nr.randint(100)):
         gene = store.rql('Any X ORDERBY RANDOM() LIMIT 1 WHERE X is Gene')[0][0]
         store.create_entity('Mutation', position_in_gene=nr.randint(1000),
@@ -144,66 +179,58 @@ def import_sequencing_results(store, subject_eid, center_eid, study_eid, platfor
                             variant_base=(u'A', u'T', u'G', u'C')[nr.randint(3)],
                             variant_frequency=100*nr.random())
 
-def import_anat_images(store, subject_eid, center_eid, study_eid, mri_1, mri_2):
+def import_anat_images(store, subject_eid, center_eid, study_eid, mri_1, mri_2, protocol_eid):
     date = random_date()
-    protocol = store.create_entity('Protocol', identifier=u'Demo anat protocol',
-                                   start_datetime=date)
-    store.relate(protocol.eid, 'related_study', study_eid, subtype='Protocol')
-    assessment = store.create_entity('Assessment', identifier=u'anat_%s' % subject_eid,
-                                     datetime=date)
-    store.relate(assessment.eid, 'protocols', protocol.eid, subtype='Assessment')
+    assessment = store.create_entity('Assessment', datetime=date)
+    store.relate(assessment.eid, 'protocols', protocol_eid, subjtype='Assessment')
     store.relate(assessment.eid, 'related_study', study_eid, subjtype='Assessment')
     store.relate(center_eid, 'holds', assessment.eid)
     store.relate(subject_eid, 'concerned_by', assessment.eid)
+    device = nr.randint(1)
     mri_data = store.create_entity('MRIData',
                                    shape_x=128, shape_y=128, shape_z=64,
                                    sequence=u'T1',
                                    voxel_res_x=1.5, voxel_res_y=1.5, voxel_res_z=1.5).eid
     scan_data = store.create_entity('Scan', has_data=mri_data,
-                                    related_study=study_eid,identifier=u'anat_%s' % subject_eid,
+                                    related_study=study_eid,
                                     label=u'anatomy', type=u'normalized T1',
                                     format=u'nii.gz',
-                                    completed=True, valid=True,
-                                    filepath=u'%s/anat.nii.gz' % subject_eid).eid
+                                    uses_device=mri_1 if device else mri_2,
+                                    completed=True, valid=True).eid
+    _file = store.create_entity('ExternalFile', name='anat_%s' % subject_eid,
+                                absolute_path=False, filepath=u'%s/anat.nii.gz' % subject_eid)
+    store.relate(scan_data, 'results_files', _file.eid)
     store.relate(scan_data, 'concerns', subject_eid, subjtype='Scan')
-    device = nr.randint(1)
-    store.relate(scan_data, 'uses_device', mri_1 if device else mri_2)
     store.relate(assessment.eid, 'generates', scan_data, subjtype='Assessment')
 
-def import_fmri_images(store, subject_eid, center_eid, study_eid, mri_1, mri_2):
+def import_fmri_images(store, subject_eid, center_eid, study_eid, mri_1, mri_2, protocol_eid):
     date = random_date()
-    protocol = store.create_entity('Protocol', identifier=u'Demo fmri protocol',
-                                   start_datetime=date)
-    store.relate(protocol.eid, 'related_study', study_eid, subtype='Protocol')
-    assessment = store.create_entity('Assessment', identifier=u'fmri_%s' % subject_eid,
-                                     datetime=date)
-    store.relate(assessment.eid, 'protocols', protocol.eid, subtype='Assessment')
+    assessment = store.create_entity('Assessment', datetime=date)
+    store.relate(assessment.eid, 'protocols', protocol_eid, subjtype='Assessment')
     store.relate(assessment.eid, 'related_study', study_eid, subjtype='Assessment')
     store.relate(center_eid, 'holds', assessment.eid)
     store.relate(subject_eid, 'concerned_by', assessment.eid)
+    device = nr.randint(1)
     mri_data = store.create_entity('MRIData',
                                    sequence=u'EPI',
                                    shape_x=128, shape_y=128, shape_z=64, shape_t=256,
                                    voxel_res_x=1.5, voxel_res_y=1.5, voxel_res_z=1.5).eid
     scan_data = store.create_entity('Scan', has_data=mri_data,
-                                    related_study=study_eid,identifier=u'fmri_%s' % subject_eid,
+                                    related_study=study_eid,
                                     label=u'bold', type=u'preprocessed fMRI',
                                     format=u'nii.gz',
-                                    completed=True, valid=True,
-                                    filepath=u'%s/bold.nii.gz' % subject_eid).eid
+                                    uses_device=mri_1 if device else mri_2,
+                                    completed=True, valid=True).eid
+    _file = store.create_entity('ExternalFile', name='bold_%s' % subject_eid,
+                                absolute_path=False, filepath=u'%s/bold.nii.gz' % subject_eid)
+    store.relate(scan_data, 'results_files', _file.eid)
     store.relate(scan_data, 'concerns', subject_eid, subjtype='Scan')
-    device = nr.randint(1)
-    store.relate(scan_data, 'uses_device', mri_1 if device else mri_2)
     store.relate(assessment.eid, 'generates', scan_data, subjtype='Assessment')
 
-def import_constrat_images(store, subject_eid, center_eid, study_eid, mri_1, mri_2):
+def import_constrat_images(store, subject_eid, center_eid, study_eid, mri_1, mri_2, protocol_eid):
     date = random_date()
-    protocol = store.create_entity('Protocol', identifier=u'Demo c map protocol',
-                                   start_datetime=date)
-    store.relate(protocol.eid, 'related_study', study_eid, subtype='Protocol')
-    assessment = store.create_entity('Assessment', identifier=u'cmap_%s' % subject_eid,
-                                     datetime=date)
-    store.relate(assessment.eid, 'protocols', protocol.eid, subtype='Assessment')
+    assessment = store.create_entity('Assessment', datetime=date)
+    store.relate(assessment.eid, 'protocols', protocol_eid, subjtype='Assessment')
     store.relate(assessment.eid, 'related_study', study_eid, subjtype='Assessment')
     store.relate(center_eid, 'holds', assessment.eid)
     store.relate(subject_eid, 'concerned_by', assessment.eid)
@@ -216,13 +243,13 @@ def import_constrat_images(store, subject_eid, center_eid, study_eid, mri_1, mri
                                            voxel_res_x=1.5, voxel_res_y=1.5, voxel_res_z=1.5).eid
             scan_data = store.create_entity('Scan', has_data=mri_data,
                                             related_study=study_eid,
-                                            identifier=u'%s_%s_%s' % (smap.replace(' ', ''),
-                                                                      ind, subject_eid),
                                             label=label, type=smap,
                                             format=u'nii.gz',
-                                            completed=True, valid=True,
-                                            filepath=u'%s/%s_%s.nii.gz' % (smap.replace(' ', ''),
-                                                                           subject_eid, ind)).eid
+                                            completed=True, valid=True).eid
+            filepath = '%s/%s_%s.nii.gz' % (smap.replace(' ', ''), subject_eid, ind)
+            _file = store.create_entity('ExternalFile', name='scan_%s_%s' % (subject_eid, ind),
+                                        absolute_path=False, filepath=filepath)
+            store.relate(scan_data, 'results_files', _file.eid)
             store.relate(scan_data, 'concerns', subject_eid, subjtype='Scan')
             store.relate(assessment.eid, 'generates', scan_data, subjtype='Assessment')
 
@@ -420,7 +447,23 @@ if __name__ == '__main__':
             score_value = store.create_entity('ScoreValue', definition=score,
                                               datetime=random_date(), value=nr.randint(_min, _max)).eid
             store.relate(subject, 'related_infos', score_value)
-
+        # Admissions
+        admdate = random_date()
+        if nr.randint(3):
+            enddate = random_date(start=admdate)
+        else:
+            enddate = None
+        admission = store.create_entity('Admission',
+                                        admission_date=admdate,
+                                        admission_end_date=enddate,
+                                        admission_of=subject,
+                                        admission_in=study)
+        for i in range(nr.randint(5)):
+            admdate = random_date(start=admdate)
+            step = store.create_entity('AdmissionStep',
+                                       step_date=admdate,
+                                       name=u'Step %s' % i,
+                                       step_of=admission.eid)
     store.flush()
     store.commit()
 
@@ -428,13 +471,15 @@ if __name__ == '__main__':
     ###############################################################################
     ### QUESTIONNAIRE RUNS ########################################################
     ###############################################################################
+    protocol_eid = create_protocol(store, study, u'Demo questionnaire protocol')
     for subject_eid in subjects:
         print subject_eid, 'questionnaire'
         import_questionnaire(store, subject_eid, center, study,
-                             ados_questionnaire_eid, ados_questions, 'ados')
+                             ados_questionnaire_eid, ados_questions,
+                             'ados', protocol_eid)
         import_questionnaire(store, subject_eid, center, study,
-                             vineland_questionnaire_eid, vineland_questions, 'vineland')
-
+                             vineland_questionnaire_eid, vineland_questions,
+                             'vineland', protocol_eid)
     store.flush()
     store.commit()
 
@@ -442,40 +487,46 @@ if __name__ == '__main__':
     ###############################################################################
     ### GENOMIC MEASURES ##########################################################
     ###############################################################################
+    protocol_eid = create_protocol(store, study, u'Demo genomics protocol')
     for subject_eid in subjects:
         print subject_eid, 'genomics'
-        measure = import_genomic(store, subject_eid, center, study, platform)
+        measure = import_genomic(store, subject_eid, center, study, platform, u'SNP', protocol_eid)
 
     store.flush()
     store.commit()
 
-    for subject_eid in subjects:
-        # Only few subject
-        if nr.randint(10):
-            print subject_eid, 'cgh'
-            measure = import_cgh_results(store, subject_eid, center, study, platform)
+    chromosome = store.rql('Any X ORDERBY RANDOM() LIMIT 1 WHERE X is Chromosome')
+    if chromosome:
+        for subject_eid in subjects:
+            # Only few subject
+            if nr.randint(10):
+                print subject_eid, 'cgh'
+                measure = import_cgh_results(store, subject_eid, center, study,
+                                             platform, protocol_eid)
+        store.flush()
+        store.commit()
 
-    store.flush()
-    store.commit()
-
-    for subject_eid in subjects:
-        # Only few subject
-        if nr.randint(10):
-            print subject_eid, 'sequencing'
-            measure = import_sequencing_results(store, subject_eid, center, study, platform)
-
-    store.flush()
-    store.commit()
+        for subject_eid in subjects:
+            # Only few subject
+            if nr.randint(10):
+                print subject_eid, 'sequencing'
+                measure = import_sequencing_results(store, subject_eid, center, study,
+                                                    platform, protocol_eid)
+        store.flush()
+        store.commit()
 
 
     ###############################################################################
     ### IMAGES MEASURES ###########################################################
     ###############################################################################
+    anat_eid = create_protocol(store, study, u'Demo anat protocol')
+    fmri_eid = create_protocol(store, study, u'Demo fmri protocol')
+    cmap_eid = create_protocol(store, study, u'Demo cmap protocol')
     for subject_eid in subjects:
         print subject_eid, 'images'
-        import_anat_images(store, subject_eid, center, study, mri_1, mri_2)
-        import_fmri_images(store, subject_eid, center, study, mri_1, mri_2)
-        import_constrat_images(store, subject_eid, center, study, mri_1, mri_2)
+        import_anat_images(store, subject_eid, center, study, mri_1, mri_2, anat_eid)
+        import_fmri_images(store, subject_eid, center, study, mri_1, mri_2, fmri_eid)
+        import_constrat_images(store, subject_eid, center, study, mri_1, mri_2, cmap_eid)
     store.flush()
     store.commit()
 
@@ -487,24 +538,42 @@ if __name__ == '__main__':
                                         'WHERE X is Disease')]
     locations = [r[0] for r in store.rql('Any X ORDERBY RANDOM() LIMIT 20 '
                                          'WHERE X is BodyLocation')]
+    techs = [r[0] for r in store.rql('Any X ORDERBY RANDOM() LIMIT 20 '
+                                     'WHERE X is MedicalTechnique')]
     drugs = [r[0] for r in store.rql('Any X ORDERBY RANDOM() LIMIT 20 '
                                          'WHERE X is Drug')]
     if diseases:
         for subject_eid in subjects:
             disease = diseases[nr.randint(len(diseases))]
             location = locations[nr.randint(len(locations))]
+            tech = techs[nr.randint(len(techs))]
+            # Diagnostic
+            diag_date = random_date()
+            quote = DR_HOUSE_QUOTES_CONCLUSIONS[nr.randint(len(DR_HOUSE_QUOTES_CONCLUSIONS))]
+            quote = u'``%s [...]`` [Dr House' % quote[:200]
+            diag = store.create_entity('Diagnostic', diagnostic_date=diag_date,
+                                       diagnostic_location=location,
+                                       diagnosed_disease=disease,
+                                       technique_type=tech,
+                                       conclusion=quote).eid
+            store.relate(subject_eid, 'related_diagnostics', diag)
+            measure = store.rql('Any X WHERE X concerns S, S eid %(e)s', {'e': subject_eid})
+            if measure:
+                store.relate(diag, 'based_on', measure[0][0])
+            # Therapy
             drug = drugs[nr.randint(len(drugs))]
             store.relate(subject_eid, 'related_diseases', disease)
             if nr.randint(3):
                 store.relate(subject_eid, 'related_lesions', location)
-            start_date = random_date()
+            start_date = random_date(start=diag_date)
             stop_date = random_date(start=start_date)
             therapy = store.create_entity('Therapy',
-                                          identifier=u'therapy_%s' % subject_eid,
-                                           start_date=start_date,
-                                           stop_date=stop_date if nr.randint(2)
+                                          start_date=start_date,
+                                          stop_date=stop_date if nr.randint(2)
                                           else None)
             store.relate(therapy.eid, 'therapy_for', disease)
+            store.relate(therapy.eid, 'based_on_diagnostic', diag)
+            store.relate(diag, 'leads_to_therapies', therapy.eid)
             store.relate(subject_eid, 'related_therapies', therapy.eid)
             stop_date = random_date(start=start_date)
             drugtake = store.create_entity('DrugTake',
@@ -513,5 +582,13 @@ if __name__ == '__main__':
                                            else None,
                                            taken_in_therapy=therapy.eid,
                                            drug=drug)
-        store.flush()
-        store.commit()
+            # TechnicalAnalysis
+            quote = DR_HOUSE_QUOTES[nr.randint(len(DR_HOUSE_QUOTES))]
+            quote = u'``%s [...]`` [Dr House' % quote[:200]
+            tech = store.create_entity('TechnicalAnalysis', analysis_date=random_date(diag_date),
+                        conclusion=quote,
+                        technique_type=techs[nr.randint(len(techs))]).eid
+            store.relate(tech, 'performed_in_therapy', therapy.eid)
+    store.flush()
+    store.commit()
+
