@@ -16,8 +16,7 @@
 # You should have received a copy of the GNU Lesser General Public License along
 # with this program. If not, see <http://www.gnu.org/licenses/>.
 
-for etype in ('Biopsy', 'BiopsySample', 'CellCulture', 'BloodSample'):
-    drop_relation_definition('TechnicalAnalysis', 'performed_on', etype)
-    add_relation_definition('TechnicalAnalysis', 'analysis_of_sample', etype)
-sync_schema_props_perms('performed_on')
-sync_schema_props_perms('measured_on_sample')
+add_cube('clinipath')
+sync_schema_props_perms('measure')
+sync_schema_props_perms('score_of_sample')
+add_relation_definition('Gene', 'relevant_gene_for', 'Study')
