@@ -50,13 +50,18 @@ class BrainomicsAdmissionView(EntityView):
         w(u'<div class="panel-heading">%s (%s %s)</div>' % (entity.dc_title(),
                                                             self._cw._('see study'),
                                                             entity.admission_in[0].view('incontext')))
-        w(u'<div class="panel-body">%s: %s %s</div>'
-          % (self._cw._('Subject age'), entity.subject_age, self._cw._('years')))
         w(u'<div class="panel-body">')
+        w(u'%s: %s %s' % (self._cw._('Subject age'), entity.subject_age, self._cw._('years')))
         w(u'<ul>')
         for step in entity.reverse_step_of:
             w(u'<li>%s</li>' % step.dc_title())
         w(u'</ul>')
+        if entity.admission_scores:
+            w(self._cw._('Scores at admission'))
+            w(u'<ul>')
+            for score in entity.admission_scores:
+                w(u'<li>%s</li>' % score.dc_title())
+            w(u'</ul>')
         w(u'</div>')
         w(u'</div>')
 
