@@ -34,7 +34,7 @@ class FilterTableLayout(TableLayout):
 
 
 def render_width(w, entity):
-    w(unicode(entity.genomic_region[0].width))
+    w(entity.genomic_region[0].width)
 
 
 class AbstractCghTableView(SlickGridEntityTableView, EntityTableView):
