@@ -184,9 +184,15 @@ _pvs.tag_object_of(('*', 'generates', 'GenomicMeasure'), 'attributes')
 _pvs.tag_subject_of(('GenomicMeasure', 'concerns', 'Subject'), 'attributes')
 _pvs.tag_subject_of(('GenomicMeasure', 'related_study', 'Study'), 'attributes')
 _pvs.tag_subject_of(('GenomicMeasure', 'chip', '*'), 'attributes')
-_pvs.tag_object_of(('*', 'related_measure', 'GenomicMeasure'), 'relations')
-_pvdc.tag_object_of(('*', 'related_measure', 'GenomicMeasure'),
-                    {'label': _('Results'), 'vid': 'genmeas-table-view'})
+_pvs.tag_object_of(('*', 'related_measure', 'GenomicMeasure'), 'hidden')
+_pvdc.display_rset('GenomicMeasure', 'relations', {'order': 2,
+                                                   'vid': 'genmeas-table-view',
+                                                   'rql': 'Any M WHERE M is Mutation, '
+                                                   'M related_measure X'})
+_pvdc.display_rset('GenomicMeasure', 'relations', {'order': 2,
+                                                   'vid': 'genmeas-table-view',
+                                                   'rql': 'Any M WHERE M is CghResult, '
+                                                   'M related_measure X'})
 _pvs.tag_object_of(('GenomicMeasure', 'chip', 'Chip'), 'relations')
 _pvdc.tag_object_of(('GenomicMeasure', 'chip', 'Chip'), {'vid': 'table'})
 _pvs.tag_object_of(('Chip', 'platform', 'GenomicPlatform'), 'relations')
@@ -200,6 +206,9 @@ _pvdc.tag_object_of(('Chip', 'platform', 'GenomicPlatform'), {'vid': 'table'})
 _pvs.tag_subject_of(('Gene', 'chromosomes', '*'), 'attributes')
 _pvdc.tag_subject_of(('Gene', 'chromosomes', '*'),
                      {'vid': 'incontext', 'label': _('Chromosomes')})
+_pvs.tag_object_of(('Gene', 'chromosomes', 'Chromosome'), 'relations')
+_pvdc.tag_object_of(('Gene', 'chromosomes', 'Chromosome'),
+                     {'vid': 'list', 'label': _('Genes')})
 # CGH results
 _pvs.tag_object_of(('*', 'genes', 'Gene'), 'hidden')
 _pvdc.display_rset('Gene', 'relations',
@@ -299,6 +308,7 @@ _pvdc.tag_object_of(('DrugTake', 'drug', '*'), {'vid': 'drugtake-table-view'})
 _pvs.tag_object_of(('*', 'related_diseases', 'Disease'), 'relations')
 _pvdc.tag_object_of(('*', 'related_diseases', 'Disease'), {'vid': 'table'})
 _pvs.tag_object_of(('Diagnostic', 'diagnosed_disease', 'Disease'), 'relations')
+_pvdc.tag_object_of(('Diagnostic', 'diagnosed_disease', 'Disease'), {'vid': 'list'})
 
 
 ###############################################################################
@@ -309,6 +319,7 @@ _pvs.tag_subject_of(('Disease', 'lesion_of', '*'), 'relations')
 _pvs.tag_subject_of(('Drug', 'acts_on', '*'), 'relations')
 _pvs.tag_subject_of(('BodyLocation', 'subpart_of', '*'), 'attributes')
 _pvs.tag_object_of(('Diagnostic', 'diagnostic_location', 'BodyLocation'), 'relations')
+_pvdc.tag_object_of(('Diagnostic', 'diagnostic_location', 'BodyLocation'), {'vid': 'list'})
 _pvs.tag_object_of(('Diagnostic', 'metastatic_locations', 'BodyLocation'), 'relations')
 
 
@@ -319,7 +330,7 @@ _pvs.tag_object_of(('Subject', 'related_samples', '*'), 'attributes')
 _pvs.tag_object_of(('AnatomicPathologyReport', 'report_on', '*'), 'relations')
 _pvdc.tag_object_of(('AnatomicPathologyReport', 'report_on', '*'), {'vid': 'clinipath-reports'})
 _pvdc.display_rset('Biopsy', 'relations', {'rql': ('Any D WHERE D is TechnicalAnalysis, '
-                                                   'D performed_on X, NOT D performed_in_therapy T'),
+                                                   'D analysis_of_sample X, NOT D performed_in_therapy T'),
                                            'vid': 'analysis-table-view',
                                            'order': 4,
                                            'label': _('Analysis')})
